@@ -45,17 +45,12 @@ information will be user_dn or user_info.
 Additional classes can be found in the config module next to this one.
 """
 
-try:
-    set
-except NameError:
-    from sets import Set as set     # Python 2.3 fallback
-
+import ldap
 import sys
 import traceback
 import pprint
 import copy
 
-import django.db
 from django.contrib.auth.models import User, Group, Permission, SiteProfileNotAvailable
 from django.core.cache import cache
 from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
@@ -66,7 +61,7 @@ try:
     from django.contrib.auth import get_user_model
     get_user_username = lambda u: u.get_username()
 except ImportError:
-    get_user_model = lambda: User
+    get_user_model = lambda: User                                        # noqa
     get_user_username = lambda u: u.username
 
 
@@ -91,7 +86,7 @@ class LDAPBackend(object):
     supports_inactive_user = False
 
     _settings = None
-    _ldap = None # The cached ldap module (or mock object)
+    _ldap = None  # The cached ldap module (or mock object)
 
     # This is prepended to our internal setting names to produce the names we
     # expect in Django's settings file. Subclasses can change this in order to
@@ -131,6 +126,14 @@ class LDAPBackend(object):
         return self._ldap
     ldap = property(_get_ldap)
 
+    def get_user_model(self):
+        """
+        By default, this will return the model class configured by
+        AUTH_USER_MODEL. Subclasses may wish to override it and return a proxy
+        model.
+        """
+        return get_user_model()
+
     #
     # The Django auth backend API
     #
@@ -149,8 +152,8 @@ class LDAPBackend(object):
         user = None
 
         try:
-            user = get_user_model().objects.get(pk=user_id)
-            _LDAPUser(self, user=user) # This sets user.ldap_user
+            user = self.get_user_model().objects.get(pk=user_id)
+            _LDAPUser(self, user=user)  # This sets user.ldap_user
         except ObjectDoesNotExist:
             pass
 
@@ -171,7 +174,7 @@ class LDAPBackend(object):
 
     def get_group_permissions(self, user, obj=None):
         if not hasattr(user, 'ldap_user') and self.settings.AUTHORIZE_ALL_USERS:
-            _LDAPUser(self, user=user) # This sets user.ldap_user
+            _LDAPUser(self, user=user)  # This sets user.ldap_user
 
         if hasattr(user, 'ldap_user'):
             return user.ldap_user.get_group_permissions()
@@ -198,7 +201,7 @@ class LDAPBackend(object):
         username is the Django-friendly username of the user. ldap_user.dn is
         the user's DN and ldap_user.attrs contains all of their LDAP attributes.
         """
-        model = get_user_model()
+        model = self.get_user_model()
         username_field = getattr(model, 'USERNAME_FIELD', 'username')
 
         kwargs = {
@@ -322,12 +325,12 @@ class _LDAPUser(object):
             user = self._user
         except self.AuthenticationFailed, e:
             logger.debug(u"Authentication failed for %s" % self._username)
-        except self.ldap.LDAPError, e:
+        except ldap.LDAPError, e:
             logger.warning(u"Caught LDAPError while authenticating %s: %s",
-                self._username, pprint.pformat(e))
+                           self._username, pprint.pformat(e))
         except Exception:
             logger.exception(u"Caught Exception while authenticating %s",
-                self._username)
+                             self._username)
             raise
 
         return user
@@ -343,9 +346,9 @@ class _LDAPUser(object):
             if self.settings.FIND_GROUP_PERMS:
                 try:
                     self._load_group_permissions()
-                except self.ldap.LDAPError, e:
+                except ldap.LDAPError, e:
                     logger.warning("Caught LDAPError loading group permissions: %s",
-                        pprint.pformat(e))
+                                   pprint.pformat(e))
 
         return self._group_permissions
 
@@ -362,12 +365,12 @@ class _LDAPUser(object):
                 self._get_or_create_user(force_populate=True)
 
             user = self._user
-        except self.ldap.LDAPError, e:
+        except ldap.LDAPError, e:
             logger.warning(u"Caught LDAPError while authenticating %s: %s",
-                self._username, pprint.pformat(e))
+                           self._username, pprint.pformat(e))
         except Exception, e:
             logger.error(u"Caught Exception while authenticating %s: %s",
-                self._username, pprint.pformat(e))
+                         self._username, pprint.pformat(e))
             logger.error(''.join(traceback.format_tb(sys.exc_info()[2])))
             raise
 
@@ -422,12 +425,12 @@ class _LDAPUser(object):
             sticky = self.settings.BIND_AS_AUTHENTICATING_USER
 
             self._bind_as(self.dn, password, sticky=sticky)
-        except self.ldap.INVALID_CREDENTIALS:
+        except ldap.INVALID_CREDENTIALS:
             raise self.AuthenticationFailed("User DN/password rejected by LDAP server.")
 
     def _load_user_attrs(self):
         if self.dn is not None:
-            search = LDAPSearch(self.dn, self.ldap.SCOPE_BASE)
+            search = LDAPSearch(self.dn, ldap.SCOPE_BASE)
             results = search.execute(self.connection)
 
             if results is not None and len(results) > 0:
@@ -449,7 +452,7 @@ class _LDAPUser(object):
 
     def _construct_simple_user_dn(self):
         template = self.settings.USER_DN_TEMPLATE
-        username = self.ldap.dn.escape_dn_chars(self._username)
+        username = ldap.dn.escape_dn_chars(self._username)
 
         self._user_dn = template % {'user': username}
 
@@ -566,8 +569,10 @@ class _LDAPUser(object):
                 logger.warning("%s does not have a value for the attribute %s", self.dn, attr)
 
     def _populate_user_from_group_memberships(self):
-        for field, group_dn in self.settings.USER_FLAGS_BY_GROUP.iteritems():
-            value = self._get_groups().is_member_of(group_dn)
+        for field, group_dns in self.settings.USER_FLAGS_BY_GROUP.iteritems():
+            if isinstance(group_dns, basestring):
+                group_dns = [group_dns]
+            value = any(self._get_groups().is_member_of(dn) for dn in group_dns)
             setattr(self._user, field, value)
 
     def _populate_and_save_user_profile(self):
@@ -616,8 +621,10 @@ class _LDAPUser(object):
         """
         save_profile = False
 
-        for field, group_dn in self.settings.PROFILE_FLAGS_BY_GROUP.iteritems():
-            value = self._get_groups().is_member_of(group_dn)
+        for field, group_dns in self.settings.PROFILE_FLAGS_BY_GROUP.iteritems():
+            if isinstance(group_dns, basestring):
+                group_dns = [group_dns]
+            value = any(self._get_groups().is_member_of(dn) for dn in group_dns)
             setattr(profile, field, value)
             save_profile = True
 
@@ -630,7 +637,7 @@ class _LDAPUser(object):
         """
         group_names = self._get_groups().get_group_names()
         groups = [Group.objects.get_or_create(name=group_name)[0] for group_name
-            in group_names]
+                  in group_names]
 
         self._user.groups = groups
 
@@ -645,9 +652,9 @@ class _LDAPUser(object):
         """
         group_names = self._get_groups().get_group_names()
 
-        perms = Permission.objects.filter(group__name__in=group_names
-            ).values_list('content_type__app_label', 'codename'
-            ).order_by()
+        perms = Permission.objects.filter(group__name__in=group_names)
+        perms = perms.values_list('content_type__app_label', 'codename')
+        perms = perms.order_by()
 
         self._group_permissions = set(["%s.%s" % (ct, name) for ct, name in perms])
 
@@ -670,9 +677,8 @@ class _LDAPUser(object):
         Binds to the LDAP server with AUTH_LDAP_BIND_DN and
         AUTH_LDAP_BIND_PASSWORD.
         """
-        self._bind_as(self.settings.BIND_DN,
-            self.settings.BIND_PASSWORD,
-            sticky=True)
+        self._bind_as(self.settings.BIND_DN, self.settings.BIND_PASSWORD,
+                      sticky=True)
 
     def _bind_as(self, bind_dn, bind_password, sticky=False):
         """
@@ -684,7 +690,7 @@ class _LDAPUser(object):
         the credentials, after which the connection will be considered unbound.
         """
         self._get_connection().simple_bind_s(bind_dn.encode('utf-8'),
-            bind_password.encode('utf-8'))
+                                             bind_password.encode('utf-8'))
 
         self._connection_bound = sticky
 
@@ -693,7 +699,7 @@ class _LDAPUser(object):
         Returns our cached LDAPObject, which may or may not be bound.
         """
         if self._connection is None:
-            self._connection = self.ldap.initialize(self.settings.SERVER_URI)
+            self._connection = ldap.initialize(self.settings.SERVER_URI)
 
             for opt, value in self.settings.CONNECTION_OPTIONS.iteritems():
                 self._connection.set_option(opt, value)
@@ -743,8 +749,10 @@ class _LDAPUserGroups(object):
 
         if self._group_names is None:
             group_infos = self._get_group_infos()
-            self._group_names = set([self._group_type.group_name_from_info(group_info)
-                for group_info in group_infos])
+            self._group_names = set(
+                self._group_type.group_name_from_info(group_info)
+                for group_info in group_infos
+            )
             self._cache_attr("_group_names")
 
         return self._group_names
@@ -777,7 +785,7 @@ class _LDAPUserGroups(object):
         """
         if self._group_dns is None:
             group_infos = self._get_group_infos()
-            self._group_dns = set([group_info[0] for group_info in group_infos])
+            self._group_dns = set(group_info[0] for group_info in group_infos)
 
         return self._group_dns
 
@@ -788,7 +796,7 @@ class _LDAPUserGroups(object):
         """
         if self._group_infos is None:
             self._group_infos = self._group_type.user_groups(self._ldap_user,
-                self._group_search)
+                                                             self._group_search)
 
         return self._group_infos
 
